@@ -32,6 +32,7 @@
 #include <sys/types.h>
 #include <netinet/in.h>
 #include <netdb.h>
+#include <pwd.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -739,6 +740,31 @@ _config_get_map (config_t *config, const char *name, int create)
   return map->config;
 }
 
+/** @internal Parse a user string.
+ * Try converting a string to a numeric user id, but if that fails,
+ * try looking it up from the passwd database.
+ *
+ * @param user is the string to convert.
+ * @param uid is the output variable.
+ *
+ * @returns Zero on success, -1 otherwise.
+ */
+static int
+_config_parse_user (const char *user, uid_t *uid)
+{
+  if (bhc_atoi (user, (long int *)uid) != 0)
+    {
+      struct passwd *pw = getpwnam (user);
+
+      if (pw == NULL)
+	return -1;
+
+      *uid = pw->pw_uid;
+      return 0;
+    }
+  return 0;
+}
+
 /** @internal Parse one option.
  * See the argp docs for details.
  */
@@ -781,7 +807,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
       cfg->userdir = bhc_strdup (arg);
       break;
     case 'U':
-      if (bhc_atoi (arg, (long int *)&global->uid) != 0)
+      if (_config_parse_user (arg, &global->uid) == -1)
 	argp_error (state, "invalid user id: `%s'", arg);
       break;
     case 'H':
@@ -804,13 +830,13 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 			  map_cfg->env, state);
       break;
     case 'i':
-      _config_parse_list (optarg, map_cfg->indexes);
+      _config_parse_list (arg, map_cfg->indexes);
       break;
     case 'C':
-      _config_parse_list (optarg, map_cfg->cgidirs);
+      _config_parse_list (arg, map_cfg->cgidirs);
       break;
     case 'c':
-      _config_parse_list (optarg, map_cfg->cgiexts);
+      _config_parse_list (arg, map_cfg->cgiexts);
       break;
     case 'p':
       if (bhc_atoi (arg, (long int *)&thy_default_port) != 0)
@@ -820,7 +846,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
       global->pidfile = bhc_strdup (arg);
       break;
     case 'e':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	{
 	  i = bhc_getsubopt (&subopts, e_subopts, &value);
@@ -851,7 +877,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	}
       break;
     case 'X':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	switch (bhc_getsubopt (&subopts, X_subopts, &value))
 	  {
@@ -901,7 +927,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	  }
       break;
     case 's':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	{
 	  int opt = bhc_getsubopt (&subopts, s_subopts, &value);
@@ -976,7 +1002,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	}
       break;
     case 'g':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	{
 	  int opt = bhc_getsubopt (&subopts, g_subopts, &value);
@@ -1020,7 +1046,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	}
       break;
     case 'r':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	switch (bhc_getsubopt (&subopts, r_subopts, &value))
 	  {
@@ -1048,7 +1074,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	  }
       break;
     case 'L':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	switch (bhc_getsubopt (&subopts, L_subopts, &value))
 	  {
@@ -1086,7 +1112,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	  }
       break;
     case 'a':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	{
 	  int opt = bhc_getsubopt (&subopts, a_subopts, &value);
@@ -1112,7 +1138,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	      global->auth.path = bhc_strdup (value);
 	      break;
 	    case THY_A_SUBOPT_UID:
-	      if (bhc_atoi (value, (long int *)&global->auth.uid) != 0)
+	      if (_config_parse_user (value, &global->auth.uid) == -1)
 		argp_error (state, "invalid Authoriser UID: `%s'", value);
 	      break;
 	    case THY_A_SUBOPT_FILE:
@@ -1126,7 +1152,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	}
       break;
     case 'W':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	{
 	  int opt = bhc_getsubopt (&subopts, W_subopts, &value);
@@ -1151,7 +1177,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	      global->worker.path = bhc_strdup (value);
 	      break;
 	    case THY_W_SUBOPT_UID:
-	      if (bhc_atoi (value, (long int *)&global->worker.uid) != 0)
+	      if (_config_parse_user (value, &global->worker.uid) == -1)
 		argp_error (state, "invalid Worker UID: `%s'", value);
 	      break;
 	    default:
@@ -1162,7 +1188,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	}
       break;
     case 'T':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	switch (bhc_getsubopt (&subopts, T_subopts, &value))
 	  {
@@ -1184,7 +1210,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	  }
       break;
     case 'o':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	switch (bhc_getsubopt (&subopts, o_subopts, &value))
 	  {
@@ -1336,7 +1362,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 		  {
 		    if (!strcasecmp (event_systems[i], value))
 		      {
-			global->options.waiter = i;
+			global->options.waiter = (thy_nqueue_waiter_t)i;
 			break;
 		      }
 		  }
@@ -1350,7 +1376,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	  }
       break;
     case 'l':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	{
 	  bhc_getsubopt (&subopts, empty_subopts, &value);
@@ -1371,7 +1397,7 @@ _config_parse_opt (int key, char *arg, struct argp_state *state)
 	map_cfg->webroot[wl - 1] = '\0';
       break;
     case 't':
-      subopts = optarg;
+      subopts = arg;
       while (*subopts != '\0')
 	switch (bhc_getsubopt (&subopts, t_subopts, &value))
 	  {
@@ -1569,12 +1595,12 @@ config_parse (int argc, char **argv)
   config->worker.args.argv = bhc_calloc (2, sizeof (char *));
 
   config->limits.post_buffer = _THY_BUFSIZE;
-  config->limits.header = 2048;
+  config->limits.header = 16384;
   config->limits.cgis = 0;
   config->limits.mmap = 0;
 
-  config->lru.fcache = _THY_LRUSIZE;
-  config->lru.ucache = _THY_LRUSIZE;
+  config->lru.fcache = 0;
+  config->lru.ucache = 0;
   config->lru.expire = _THY_LRUEXPIRE;
 
   cfg.global = config;
