@@ -110,13 +110,24 @@ _session_parse_absoluteuri (session_t *session)
 
   if (strncmp ("http://", session->request->url, 7))
     return;
+  if (strlen (session->request->url) <= 7)
+    return;
 
   tmp1 = strstr (session->request->url, "http://");
   tmp2 = bhc_strdup (strchr (&tmp1[7], '/'));
   if (tmp2)
     {
-      size_t l = strlen (tmp1);
-      tmp1[l - strlen (tmp2)] = 0;
+      size_t l1 = strlen (tmp1);
+      size_t l2 = strlen (tmp2);
+      size_t l = l1 - l2;
+
+      if (l <= 7)
+	{
+	  free (tmp2);
+	  return;
+	}
+
+      tmp1[l] = 0;
       free (session->request->host);
       session->request->host = bhc_strdup (&tmp1[7]);
       free (session->request->url);
@@ -1429,20 +1440,8 @@ session_handle (session_t *session)
       if (!session->request->host)
 	{
 	  free (session->request->resolved);
-	  tmp = fabs_urlmap (session->request->url, "default",
-			     session->absuri);
-	  if (!(session->request->resolved = fabs_realpath (tmp)))
-	    {
-	      free (tmp);
-	      tmp = fabs_urlmap (session->request->url, NULL,
-				 session->absuri);
-	      session->request->resolved = fabs_realpath (tmp);
-	      session->root = bhc_strdup (map_config->webroot);
-	    }
-	  else
-	    asprintf (&session->root, "%s/%s", map_config->webroot,
-		      "default");
-	  free (tmp);
+	  asprintf (&session->root, "%s/%s", map_config->webroot,
+		    "default");
 	}
       else
 	{
@@ -1550,16 +1549,19 @@ session_handle (session_t *session)
       session->request->url[1] == '~')
     {
       char *ud = userdir (session->request->url, map_config->userdir);
-      if (!strstr (session->request->resolved, ud))
+      if (ud != NULL)
 	{
-	  if (!follow_ifowner (session))
+	  if (!strstr (session->request->resolved, ud))
 	    {
-	      free (ud);
-	      free (map_config);
-	      return _session_error_end (session, HTTP_STATUS_404);
+	      if (!follow_ifowner (session))
+		{
+		  free (ud);
+		  free (map_config);
+		  return _session_error_end (session, HTTP_STATUS_404);
+		}
 	    }
+	  free (ud);
 	}
-      free (ud);
     }
   else
     {
